// ==============================================================
// Here is an example showing how to use ACCBOOK.DLL in order to
// get moves and statistical data from the books generated by
// PgnScanner. Actually, it gives you the same powerful features
// as ChessBase trees in your own engine without having to write
// boring books and openings code. If you want to use statistical
// data such as elo or perf., you have to generate a book based
// on PGN with elo ratings before. If your PGN games don't have
// elo ratings, PgnScanner can calculate them with the 'givelo'
// command. Please, see help online for more informations.
// ==============================================================
// You can freely use PgnScanner, books and accbook.dll for non-
// commercial use only. However, a link referring to the author
// would be appreciated. If you want to distribute it in a
// commercial chess package, you must contact me before :
// Gabriel Guillory,     bal5(NO@SPAM)club-internet.fr
// http://www.transversale.fr
// ==============================================================
//
//
// ===============================================
// ACCBOOK.DLL puts 5 functions at your disposal :
// ===============================================
// LoadBook(void)
// GetMoves(char *epdString, char *mvBuf, unsigned long bufSz, bool log)
// GetAMove(int mode, char *epdString, char *mvBuf, unsigned long bufSz, bool log)
// GetData(char *epdString, unsigned int tabSz, char *mv, unsigned long *nGm,
//         unsigned int *pc, unsigned int *myElo, unsigned int *perf, unsigned int *cmt)
// CloseBook(void)
//
//
// =============================
// The parameters required are :
// =============================
// epdString :	epd or fen string of the position you want to search for
// mvBuf :		string buffer to get found moves
// bufSz :		size of mvBuf (in bytes) => 5 bytes as a minimum for GetAMove
// log :		if you want to save found moves and info in the log file 
// mode :		type of move choice :
//				1 = auto, 2 = greatest nb of game, 3 = max elo gain (opp elo <> perf)
//
//				note : "auto" use the probability P to chose the move in the list according to :
//				P(i) = nbOfGames(i) / SUM(nbOfGames(i) ... nbOfGames(max))
//					P as the probability to chose the move i
//					i as the rank of the move in the resulting move-list sorted by number of games
//				>>in auto mode, no moves are choosen if it remains less than 3 games for the top move
//					
// tabSz :		maximum length of the destination table. It corresponds to the maximum
//				number of moves that any position could have.
// mv :			a pointer to a "char mv[tabSz][5]" table for the moves
// nGm :		a pointer to a "unsigned long nGm[tabSz]" table for the "nb of game" informations
// pc :			a pointer to a "unsigned int pc[tabSz]" table for the percentage informations
// myElo :		a pointer to a "unsigned int myElo[tabSz]" table for the myElo informations
// perf :		a pointer to a "unsigned int perf[tabSz]" table for the performance informations
// cmt :		a pointer to a "unsigned int cmt[tabSz]" table for the auto evaluated comments



#include <stdio.h>
#include <windows.h>


//+-----------------------------+
//| write these typedef to make |
//| your functions easy to read |
//+-----------------------------+
typedef int (pascal *FUNC1)(void);
typedef int (pascal *FUNC2)(char *epdString, char *mvBuf, unsigned long bufSz, bool logMoves);
typedef int (pascal *FUNC3)(void);
typedef int (pascal *FUNC4)(int mode, char *epdString, char *mvBuf, unsigned long bufSz, bool logMoves);
typedef int (pascal *FUNC5)(char *epdString, unsigned int tabSz, char *mv, unsigned long *nGm,
	         unsigned int *pc, unsigned int *myElo, unsigned int *perf, unsigned int *cmt);


//+-------------+
//| Entry point |
//+-------------+
int main(void) {
	
	HINSTANCE	h;
	FUNC1		LoadBook;
	FUNC2		GetMoves;
	FUNC3		CloseBook;
	FUNC4		GetAMove;
	FUNC5		GetData;

	h = LoadLibrary("accbook.dll");

	if (h) {

		LoadBook = (FUNC1)GetProcAddress(h, "LoadBook");	// load book.psb (64 bits & num:224 required !!)
		GetMoves = (FUNC2)GetProcAddress(h, "GetMoves");	// enter EPD string and get moves string
		CloseBook = (FUNC3)GetProcAddress(h, "CloseBook");	// free memory
		GetAMove = (FUNC4)GetProcAddress(h, "GetAMove");	// get a move from the book
		GetData = (FUNC5)GetProcAddress(h, "GetData");		// get stat data in own format

		if (LoadBook()) {

			//-----------------------------------
			// Once the book is loaded, positions
			// are instantaneously found. Here
			// are some various easy examples :
			//-----------------------------------
			char	mvList[1024];
			char	singlMv[5];
			int		i, n;

			//-----------
			// Example #1
			//-----------
			printf("\nExample #1\nPress enter to get a moves list + stats in string format :\n");
			getchar();
			printf("After the start position :\n\n");
			GetMoves("rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w" ,mvList ,1024 ,true);
			printf(mvList);

			//-----------
			// Example #2
			//-----------
			printf("\n\nExample #2\nPress enter to get a single move in auto mode (1)\n");
			getchar();
			printf("After 1.d4 :\n\n");
			GetAMove(1, "rnbqkbnr/pppppppp/8/8/3P4/8/PPP1PPPP/RNBQKBNR b" ,singlMv ,20 ,true);
			printf(singlMv);			
			
			//-----------
			// Example #3
			//-----------
			const int		sz = 128;
			char			mv[1024];	// moves have always a size of 5 char and are
										// stored with a null at each 5th char, just like
										// "e2e4\0d2d4\0c2c4\0g1??\0????\0". So you have
										// to do something like mv+(i*5) to access the i-move
			unsigned long	nGm[sz];
			unsigned int	pc[sz];
			unsigned int	myElo[sz];
			unsigned int	perf[sz];
			unsigned int	cmt[sz];
						
			printf("\n\n\nExample #3\nPress enter to get a moves list + stats in >real< format\n");
			getchar();
			printf("After 1.d4 :\n\n");
			n = GetData("rnbqkbnr/pppppppp/8/8/3P4/8/PPP1PPPP/RNBQKBNR b", sz, mv, nGm, pc, myElo, perf, cmt);
			for (i = 0; i < n; i++) {
				printf("%d.%s  nbGames=%d  score=%d%%  myElo=%d  perf=%d  comment=%d [1-7]\n"
						, i+1, mv+(5*i), nGm[i], pc[i], myElo[i], perf[i], cmt[i]);
			}

			//--------
			// the end
			//--------
			printf("\nThat's the end...\n");
			getchar();
			
			CloseBook();
		}
		FreeLibrary(h);
	}
	
	// please see accbook.log to see processes or errors
	return 1;
}